/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.pagesubscription;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;
import java.util.regex.Pattern;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Value;

import net.coobird.thumbnailator.tasks.UnsupportedFormatException;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.generation.ServiceableGenerator;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.xml.sax.SAXException;

import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.RepositoryConstants;
import org.ametys.plugins.repository.jcr.JCRAmetysObject;
import org.ametys.runtime.user.User;
import org.ametys.runtime.user.UsersManager;
import org.ametys.web.WebConstants;
import org.ametys.web.repository.page.Page;

/**
 * Generate user info about a page
 */
public class InfoSubscriberGenerator extends ServiceableGenerator
{
    /** Constant for the attachment node name. */
    public static final String SUBSCRIBERS_PROPERTY_NAME = RepositoryConstants.NAMESPACE_PREFIX_INTERNAL + ":subscriberMails";
    
    /** The pattern to check emails */
    protected static final Pattern EMAIL_VALIDATOR = Pattern.compile("^([a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-zA-Z0-9]{2,})?$");
    
    /** The pattern to check text input */
    protected static final Pattern TEXT_VALIDATOR = Pattern.compile("^\\s*$");
    
    /** Repository content */
    protected AmetysObjectResolver _resolver;
    
    /** FO user manager */
    protected UsersManager _foUsersManager;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
        _foUsersManager = (UsersManager) smanager.lookup(UsersManager.ROLE + ".FO");
    }

    public void generate() throws IOException, SAXException, ProcessingException
    {
        Request request = ObjectModelHelper.getRequest(objectModel);
        
        String login = (String) request.getAttribute(WebConstants.FO_LOGIN);
        User user = _foUsersManager.getUser(login);
        
        contentHandler.startDocument();
        AttributesImpl attr = new AttributesImpl();
        if (user == null)
        {
            attr.addCDATAAttribute("isLog", "false");
            XMLUtils.createElement(contentHandler, "user", attr);
        }
        else
        {
            attr.addCDATAAttribute("isLog", "true");
            String email = user.getEmail();
          
            String pageId = request.getParameter("page-id");
            if (pageId == null) 
            {
                throw new IllegalStateException("Unable to subscribe or unsubscribe to the page: cannot determine the current page");
            }
            else
            {
                Page page = _resolver.resolveById(pageId);
                if (page instanceof JCRAmetysObject)
                {
                    Set<String> subscribers = _getSubscribers((JCRAmetysObject) page);
                    attr.addCDATAAttribute("isSubscribe", Boolean.toString(subscribers.contains(email)));
                }
                else
                {
                    throw new UnsupportedFormatException("Unable to subscribe or unsubscribe to the page: the page is not subscribable");
                }
            }
            XMLUtils.createElement(contentHandler, "user", attr, email);    
        }
        contentHandler.endDocument();
    }

    private Set<String> _getSubscribers (JCRAmetysObject page)
    {
        Set<String> subscriberMails = new HashSet<String>();
        
        try
        {
            Node node = page.getNode();
            if (node.hasProperty(SUBSCRIBERS_PROPERTY_NAME))
            {
                Value[] values = node.getProperty(SUBSCRIBERS_PROPERTY_NAME).getValues();
                for (Value value : values)
                {
                    subscriberMails.add(value.getString());
                }
            }
        }
        catch (RepositoryException e)
        {
            getLogger().error("Unable to retrieve subscribers for page of id'" + page.getId() + "'", e);
        }
        
        return subscriberMails;
    }

}
