/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.pagesubscription;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Value;

import net.coobird.thumbnailator.tasks.UnsupportedFormatException;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.acting.ServiceableAction;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.commons.lang.StringUtils;

import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.RepositoryConstants;
import org.ametys.plugins.repository.jcr.JCRAmetysObject;
import org.ametys.runtime.util.I18nizableText;
import org.ametys.runtime.util.cocoon.ActionResultGenerator;
import org.ametys.web.repository.page.Page;
import org.ametys.web.repository.page.jcr.DefaultPage;

/**
 * Action to subscribe to a page
 */
public class PageSubscribeAction extends ServiceableAction
{
    /** Constant for the attachment node name. */
    public static final String SUBSCRIBERS_PROPERTY_NAME = RepositoryConstants.NAMESPACE_PREFIX_INTERNAL + ":subscriberMails";
    
    /** The pattern to check emails */
    protected static final Pattern EMAIL_VALIDATOR = Pattern.compile("^([a-z0-9._-]+@[a-z0-9.-]{2,}[.][a-zA-Z0-9]{2,})?$");
    
    /** The pattern to check text input */
    protected static final Pattern TEXT_VALIDATOR = Pattern.compile("^\\s*$");
    
    /** Repository content */
    protected AmetysObjectResolver _resolver;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
    }
    
    public Map act(Redirector redirector, SourceResolver resolver, Map objectModel, String source, Parameters parameters) throws Exception
    {
        Map<String, Object> result = new HashMap<String, Object>();
        List<I18nizableText> errors = new ArrayList<I18nizableText>();
        
        Request request = ObjectModelHelper.getRequest(objectModel);
        String action = request.getParameter("page-subscribe-action");
        result.put("action", action);
        
        String email = request.getParameter("email");
        
        if (email == null || !EMAIL_VALIDATOR.matcher(StringUtils.trimToEmpty(email)).matches() || TEXT_VALIDATOR.matcher(StringUtils.trimToEmpty(email)).matches())
        {
            errors.add(new I18nizableText("plugin.page-subscription", "PLUGINS_PAGE_SUBSCRIBE_FORM_MAIL_ERROR_MAILBY"));
        }
        else
        {
            result.put("email", email);
        }
        
        String pageId = request.getParameter("page-id");
        if (pageId == null) 
        {
            throw new IllegalStateException("Unable to subscribe or unsubscribe to the page: cannot determine the current page");
        }
        else
        {
            Page page = _resolver.resolveById(pageId);
            
            if (page == null)
            {
                throw new IllegalStateException("Unable to subscribe or unsubscribe to the page: the page of id " + pageId + " is unknown");
            }
            
            if (page instanceof JCRAmetysObject)
            {
                Set<String> subscribers = _getSubscribers((JCRAmetysObject) page);
                
                if (action.equals("subscribe"))
                {
                    subscribers.add(email);
                }
                else if (action.equals("unsubscribe"))
                {
                    subscribers.remove(email);
                }
                
                _setSubscribers((DefaultPage) page, subscribers);
            }
            else
            {
                throw new UnsupportedFormatException("Unable to subscribe or unsubscribe to the page: the page is not subscribable");
            }
        }
          
        result.put("error", errors);
        request.setAttribute(ActionResultGenerator.MAP_REQUEST_ATTR, result);
        
        return EMPTY_MAP;
    }
    
    private Set<String> _getSubscribers (JCRAmetysObject page)
    {
        Set<String> subscriberMails = new HashSet<String>();
        
        try
        {
            Node node = page.getNode();
            if (node.hasProperty(SUBSCRIBERS_PROPERTY_NAME))
            {
                Value[] values = node.getProperty(SUBSCRIBERS_PROPERTY_NAME).getValues();
                for (Value value : values)
                {
                    subscriberMails.add(value.getString());
                }
            }
        }
        catch (RepositoryException e)
        {
            getLogger().error("Unable to retrieve subscribers for page of id'" + page.getId() + "'", e);
        }
        
        return subscriberMails;
    }
    
    private void _setSubscribers (DefaultPage page, Set<String> subscribers)
    {
        try
        {
            Node node = page.getNode();
            
            String[] subscribersAsStringArray = subscribers.toArray(new String[subscribers.size()]);
            node.setProperty(SUBSCRIBERS_PROPERTY_NAME, subscribersAsStringArray);
            page.saveChanges();
        }
        catch (RepositoryException e)
        {
            getLogger().error("Unable to set subscribers for page of id'" + page.getId() + "'", e);
        }
    }

}
