/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.pagesubscription.helpers;

import java.util.HashSet;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Value;

import net.coobird.thumbnailator.tasks.UnsupportedFormatException;

import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.logger.Logger;
import org.apache.cocoon.components.ContextHelper;
import org.apache.cocoon.environment.Request;

import org.ametys.plugins.repository.RepositoryConstants;
import org.ametys.plugins.repository.jcr.JCRAmetysObject;
import org.ametys.web.repository.page.Page;

/**
 *  Helper component to be used from XSL stylesheets.
 */
public class PageSubscriptionXSLTHelper implements LogEnabled, Component, Contextualizable
{
    /** Constant for the attachment node name. */
    public static final String SUBSCRIBERS_PROPERTY_NAME = RepositoryConstants.NAMESPACE_PREFIX_INTERNAL + ":subscriberMails";
    
    private static Logger _logger;
    private static Context _context;
    
    @Override
    public void contextualize(Context context) throws ContextException
    {
        _context = context;
    }
    
    @Override
    public void enableLogging(Logger logger)
    {
        _logger = logger;
    }
    
    /**
     * Determines is the given email is registered as subscriber on current page
     * @param email The email
     * @return true if the given email is registered as subscriber on current page
     * @throws UnsupportedFormatException if the current page is not a JCRPage
     */
    public static boolean isPageSubscriber(String email) throws UnsupportedFormatException
    {
        Request request = ContextHelper.getRequest(_context);
        Page page = (Page) request.getAttribute(Page.class.getName());
        
        if (page instanceof JCRAmetysObject)
        {
            Set<String> subscribers = _getSubscribers((JCRAmetysObject) page);
            return subscribers.contains(email);
        }
        else
        {
            throw new UnsupportedFormatException("Unable to subscribe or unsubscribe to the page: the page is not subscribable");
        }          
    }
    
    /**
     * Get the page's subscribers
     * @param page the page
     * @return the registered emails
     */
    protected static Set<String> _getSubscribers (JCRAmetysObject page)
    {
        Set<String> subscriberMails = new HashSet<String>();
        
        try
        {
            Node node = page.getNode();
            if (node.hasProperty(SUBSCRIBERS_PROPERTY_NAME))
            {
                Value[] values = node.getProperty(SUBSCRIBERS_PROPERTY_NAME).getValues();
                for (Value value : values)
                {
                    subscriberMails.add(value.getString());
                }
            }
        }
        catch (RepositoryException e)
        {
            _logger.error("Unable to retrieve subscribers for page of id'" + page.getId() + "'", e);
        }
        
        return subscriberMails;
    }

}
