/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.pagesubscription.observer;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.mail.MessagingException;

import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.logger.Logger;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;

import org.ametys.plugins.explorer.observation.ExplorerEvent;
import org.ametys.plugins.explorer.observation.ExplorerEvent.EventType;
import org.ametys.plugins.explorer.observation.ExplorerListener;
import org.ametys.plugins.explorer.resources.Resource;
import org.ametys.plugins.explorer.resources.jcr.JCRResource;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.RepositoryConstants;
import org.ametys.plugins.repository.jcr.JCRAmetysObject;
import org.ametys.runtime.config.Config;
import org.ametys.runtime.plugins.core.mail.SendMailHelper;
import org.ametys.runtime.util.I18nUtils;
import org.ametys.runtime.util.I18nizableText;
import org.ametys.runtime.util.URLEncoder;
import org.ametys.web.repository.page.Page;

/**
 * Listen attachment page and send mail
 */
public class PageSubscriptionExplorerListener implements ExplorerListener, Serviceable, LogEnabled
{
    /** Constant for the attachment node name. */
    public static final String SUBSCRIBERS_PROPERTY_NAME = RepositoryConstants.NAMESPACE_PREFIX_INTERNAL + ":subscriberMails";

    private AmetysObjectResolver _resolver;
    private Logger _logger;
    private I18nUtils _i18nUtils;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
        _i18nUtils = (I18nUtils) smanager.lookup(I18nUtils.ROLE);
    }
    
    @Override
    public void enableLogging(Logger logger)
    {
        _logger = logger;
    }
    
    public void onEvent(ExplorerEvent event)
    {
        EventType type = event.getEventType();
        
        switch (type)
        {
            case RESOURCE_CREATE:
            case RESOURCE_UPDATE:
            case RESOURCE_DELETE:
                _doObserve(event);
                break;
            default:
                break;
        }
    }
    
    /**
     * Listen resource event
     * @param event
     */
    protected void _doObserve (ExplorerEvent event)
    {
        Page page = _getPage (event);
        if (page != null)
        {
            Set<String> subscribers = _getSubscribers((JCRAmetysObject) page);
            if (!subscribers.isEmpty())
            {
                String textBody = _getMailBody(event, page);
                String subject = _getMailSubject(event.getEventType(), page);
                
                String sender = Config.getInstance().getValueAsString("smtp.mail.from");
                
                for (String subscriberMail : subscribers)
                {
                    try
                    {
                        SendMailHelper.sendMail(subject, null, textBody, subscriberMail, sender);
                    }
                    catch (MessagingException e)
                    {
                        _logger.error("Unable to send a mail to '" + "toto");
                    }
                }     
            }

        }
    }
    
    private String _getAbsolutePageUrl (Page page)
    {
        return new StringBuilder().append(page.getSite().getUrl())
                .append("/").append(page.getSitemapName())
                .append("/").append(page.getPathInSitemap())
                .append(".html")
                .toString();
    }
    
    private String _getAbsoluteResourceUrl (Resource resource, Page page)
    {
        return new StringBuilder().append(page.getSite().getUrl())
                .append("/").append(page.getSitemapName())
                .append("/").append(page.getPathInSitemap())
                .append("/_attachments/")
                .append(_encodePath(resource.getResourcePath()))
                .append("?download=true")
                .toString();
    }
    
    /**
     * Encode the resource path
     * @param path the resource path
     * @return the encoded URI
     */
    protected String _encodePath (String path)
    {
        StringBuffer sb = new StringBuffer();
        
        String[] parts = path.split("/");
        for (String part : parts)
        {
            sb.append("/");
            sb.append(URLEncoder.encode(part));
        }
        return sb.toString();
    }
    
    /**
     * Get the page's subscribers
     * @param page the page
     * @return the registered emails
     */
    private Set<String> _getSubscribers (JCRAmetysObject page)
    {
        Set<String> subscriberMails = new HashSet<String>();
        
        try
        {
            Node node = page.getNode();
            if (node.hasProperty(SUBSCRIBERS_PROPERTY_NAME))
            {
                Value[] values = node.getProperty(SUBSCRIBERS_PROPERTY_NAME).getValues();
                for (Value value : values)
                {
                    subscriberMails.add(value.getString());
                }
            }
        }
        catch (RepositoryException e)
        {
            _logger.error("Unable to retrieve subscribers for page of id'" + page.getId() + "'", e);
        }
        
        return subscriberMails;
    }
    
    /**
     * Create the mail subject depend on the event
     * @param eventType
     * @param page
     * @return the mail subject
     */
    protected String _getMailSubject (EventType eventType, Page page)
    {
        List<String> i18nparam = new ArrayList<String>();
        i18nparam.add(page.getTitle());
        
        I18nizableText i18nSubject = null;
        switch (eventType)
        {
            case RESOURCE_CREATE:
                i18nSubject = new I18nizableText("plugin.page-subscription", "PLUGINS_PAGE_SUBSCRIBE_SEND_MAIL_CREATE_TITLE", i18nparam);
                break;
            case RESOURCE_UPDATE:
                i18nSubject = new I18nizableText("plugin.page-subscription", "PLUGINS_PAGE_SUBSCRIBE_SEND_MAIL_UPDATE_TITLE", i18nparam);
                break;
            case RESOURCE_DELETE:
                i18nSubject = new I18nizableText("plugin.page-subscription", "PLUGINS_PAGE_SUBSCRIBE_SEND_MAIL_DELETE_TITLE", i18nparam);
                break;
            default:
                break;
        }                
        
        String subject = _i18nUtils.translate(i18nSubject);
        return subject;
    }   
        
    /**
     * Create the mail body depend on the event
     * @param event
     * @param page
     * @return the mail body
     */
    protected String _getMailBody (ExplorerEvent event, Page page)
    {
        List<String> i18nparam = new ArrayList<String>();
        
        i18nparam.add(_getAbsolutePageUrl(page));
        i18nparam.add(page.getTitle());
        i18nparam.add(page.getSite().getTitle());
        
        I18nizableText i18nTextBody = null;
        JCRResource resource = null;
        String resourceId = null;
        switch (event.getEventType())
        {
            case RESOURCE_CREATE:
                resourceId = event.getTarget();
                resource = _resolver.resolveById(resourceId);
                
                i18nparam.add(resource.getName());
                i18nparam.add(_getAbsoluteResourceUrl(resource, page));
                
                i18nTextBody = new I18nizableText("plugin.page-subscription", "PLUGINS_PAGE_SUBSCRIBE_SEND_MAIL_CREATE_TEXT_BODY", i18nparam);
                break;
            case RESOURCE_UPDATE:
                resourceId = event.getTarget();
                resource = _resolver.resolveById(resourceId);
                
                i18nparam.add(resource.getName());
                i18nparam.add(_getAbsoluteResourceUrl(resource, page));
                
                i18nTextBody = new I18nizableText("plugin.page-subscription", "PLUGINS_PAGE_SUBSCRIBE_SEND_MAIL_UPDATE_TEXT_BODY", i18nparam);
                break;
            case RESOURCE_DELETE:
                
                i18nparam.add((String) event.getParameters().get("name"));
                
                i18nTextBody = new I18nizableText("plugin.page-subscription", "PLUGINS_PAGE_SUBSCRIBE_SEND_MAIL_DELETE_TEXT_BODY", i18nparam);
                break;
            default:
                break;
        }                
        String textBody = _i18nUtils.translate(i18nTextBody);
        return textBody;
    }
    
    /**
     * Get the page linked to the event
     * @param event
     * @return the page linked to the event
     */
    protected Page _getPage (ExplorerEvent event)
    {
        AmetysObject object = null;
        if (event.getEventType().equals(EventType.RESOURCE_DELETE))
        {
            String parentId = (String) event.getParameters().get("parentID");
            object = _resolver.resolveById(parentId);
        }
        else
        {
            String targetId = event.getTarget();
            object = _resolver.resolveById(targetId);
        }
        
        AmetysObject parent = object.getParent();
        while (parent != null)
        {
            if (parent instanceof Page)
            {
                return (Page) parent;
            }
            parent = parent.getParent();
        }
        return null;
    }

}
